import type { StockDatabase, StockKey } from "./interfaces.js";
import type { Aggregate } from "../interfaces.js";
import { clickhouse, query } from "../../lib/clickhouse.js";

function makeStockDatabase(): StockDatabase {
  const stockDatabase: Omit<StockDatabase, "getSymbols"> = {
    getKeys: async ({ date, key }) => {
      if (key?.symbol) {
        return [key as StockKey];
      }
      return await query(`
          SELECT DISTINCT symbol FROM stock_aggregates WHERE toDate(tsStart) = '${date}'
        `);
    },
    getAggregates: async ({ key: { symbol }, date }) => {
      return (
        await query<Omit<Aggregate<StockKey>, "key">>(`
        SELECT 
            toUnixTimestamp(tsStart) as tsStart,
            open,
            close,
            high,
            low
        FROM stock_aggregates
        WHERE symbol = '${symbol}'
        AND toDate(tsStart) = '${date}'
        ORDER BY tsStart ASC
      `)
      ).map((aggregate) => ({
        ...aggregate,
        tsStart: aggregate.tsStart * 1000, // unfortunately, `toUnixTimestamp` only returns second-precision
      }));
    },
    getAggregate: async ({ key: { symbol }, tsStart }) => {
      return (
        await query<Omit<Aggregate<StockKey>, "key">>(`
          SELECT 
            open,
            close,
            high,
            low
          FROM stock_aggregates
          WHERE symbol = '${symbol}'
          AND tsStart = '${tsStart}'
        `)
      ).map((aggregate) => ({
        ...aggregate,
        tsStart,
      }))[0];
    },
    insertAggregates: async (aggregates) => {
      // stock existence is taken care of by clickhouse materialized view
      await clickhouse.insert({
        table: "stock_aggregates",
        values: aggregates.map(
          ({ key: { symbol }, tsStart, open, close, high, low }) => ({
            symbol,
            tsStart,
            open,
            close,
            high,
            low,
          })
        ),
      });
    },
    getClosingPrice: async ({ key }) => {
      // no-op: not used since stocks don't have a "closing" price, unlike options.
      return 0;
    },
  };

  return {
    ...stockDatabase,
    getSymbols: stockDatabase.getKeys,
  };
}

export const database: StockDatabase = makeStockDatabase();
