import { query } from "./lib/clickhouse.js";
import { publicProcedure, RpcType, router } from "./trpc.js";
import {
  Object as ObjectT,
  String as StringT,
  Number as NumberT,
} from "@sinclair/typebox";

/** Returns prices for all matching calendars (i.e. those with similar
 * characteristics to those given) */
export const getChartData = publicProcedure
  .input(
    RpcType(
      ObjectT({
        underlying: StringT({ maxLength: 5 }),
        daysToFrontExpiration: NumberT(),
        daysBetweenFrontAndBackExpiration: NumberT(),
        strikePercentageFromUnderlyingPriceRangeMin: NumberT(),
        strikePercentageFromUnderlyingPriceRangeMax: NumberT(),
        lookbackPeriodStart: StringT(),
        lookbackPeriodEnd: StringT(),
      })
    )
  )
  .query(async (opts) => {
    const {
      underlying,
      daysToFrontExpiration,
      daysBetweenFrontAndBackExpiration,
      strikePercentageFromUnderlyingPriceRangeMin,
      strikePercentageFromUnderlyingPriceRangeMax,
      lookbackPeriodStart,
      lookbackPeriodEnd,
    } = opts.input;
    return await query<[number, number]>(
      `
		 SELECT
			 toUnixTimestamp(tsStart) as x,
			 truncate(calendarPrice, 2) as y
		 FROM calendar_histories
		 WHERE symbol = '${underlying}'
		 AND daysToFrontExpiration = ${daysToFrontExpiration}
		 AND strikePercentageFromUnderlyingPrice >= ${strikePercentageFromUnderlyingPriceRangeMin}
		 AND strikePercentageFromUnderlyingPrice <= ${strikePercentageFromUnderlyingPriceRangeMax}
		 AND daysBetweenFrontAndBackExpiration = ${daysBetweenFrontAndBackExpiration}
		 AND tsStart >= '${lookbackPeriodStart} 00:00:00'
		 AND tsStart <= '${lookbackPeriodEnd} 00:00:00'
	 `,
      "JSONEachRow"
    );
  });

export default router({
  getChartData,
});
