import { createClient as createClickhouseClient } from "@clickhouse/client";
import type { DataFormat } from "@clickhouse/client";
import { Env } from "@humanwhocodes/env";
import { retry } from "./utils/retry.js";

const env = new Env();

const { CLICKHOUSE_USER, CLICKHOUSE_PASS } = env.required;
const CLICKHOUSE_HOST = env.get("CLICKHOUSE_HOST", "http://localhost:8123");

export const clickhouse = createClickhouseClient({
  url: CLICKHOUSE_HOST,
  username: CLICKHOUSE_USER,
  password: CLICKHOUSE_PASS,
  keep_alive: {
    enabled: true,
    // socket_ttl: 2500,
  },
});

export async function query<T>(
  queryString: string,
  format: DataFormat = "JSONEachRow"
): Promise<Array<T>> {
  return await retry(
    async () => {
      const result = await clickhouse.query({
        query: queryString,
        format,
        clickhouse_settings: {
          output_format_json_quote_64bit_integers: 0,
        },
      });
      return await result.json();
    },
    { maxRetries: 5 }
  );
}
